Program MPI_IO_Test

  Use MPI
  Implicit None

  Integer, Parameter :: wp=Kind(0.0D0)

  Integer :: myid, myxid, myyid
  Integer :: numprocs
  Integer :: ierror, i

  Integer, Parameter :: g_nx=40000
  Integer, Parameter :: g_ny=20000

  ! Create a global size array for constructing a MPI Subarray datatype

  Integer, Dimension(2), Parameter :: global_size=(/ g_nx, g_ny /)

  Integer :: nxproc, nyproc
  Integer :: l_nx, l_ny

  Real(wp), Dimension(:,:), Allocatable :: localTData

  Integer :: status
  Integer :: ncid
  Integer :: dimids(2)
  Integer :: temp_id
  Integer :: starts(2), counts(2), strides(2)
  Integer :: sizes(2), subsizes(2), begins(2)

  Integer :: filehandle

  Integer :: standard_local_size(2), local_size(2), local_offset_in_global(2)

  Integer(Kind=MPI_OFFSET_KIND) :: offset=0

  Integer :: subarray, nonhaloarray

  !
  ! MPI init
  !
  
  Call MPI_Init( ierror )
  Call MPI_Comm_size( MPI_COMM_WORLD, numprocs, ierror )
  Call MPI_Comm_rank( MPI_COMM_WORLD, myid, ierror )
  
  nxproc = 16
  nyproc = 12
  
  myxid = Mod( myid, nxproc )
  myyid = myid/nxproc
  
  If( nxproc*nyproc .ne. numprocs )then
     print *, 'numprocs != nxproc*nyproc'
     stop
  End If
  
  l_nx = (g_nx+nxproc-1)/nxproc
  l_ny = (g_ny+nyproc-1)/nyproc

  standard_local_size=(/ l_nx, l_ny /)

  ! Define where in the global array we begin

  local_offset_in_global = (/ myxid*l_nx, myyid*l_ny /)

  ! Set the local size array

  local_size = (/ l_nx, l_ny /)

  ! Possibly adjust the array sizes on the last processor

  Do i=1,2
     If(local_offset_in_global(i)+local_size(i)>global_size(i))Then
        local_size(i)=global_size(i)-local_offset_in_global(i)
     End If
  End Do

  !
  ! Create local arrays with halo region of 1 cell
  !
  
  Allocate( localTData(0:local_size(1)+1,0:local_size(2)+1) )
  
  localTData = Real(100+myid+1,wp)
  
!  Call MPI_File_open(MPI_COMM_WORLD,"mydata.dat",MPI_MODE_RDWR+MPI_MODE_CREATE+&
!         &MPI_MODE_EXCL,MPI_INFO_NULL,filehandle,ierror)
  Call MPI_File_open(MPI_COMM_WORLD,"mydata.dat",MPI_MODE_RDWR+MPI_MODE_CREATE,&
         &MPI_INFO_NULL,filehandle,ierror)

  ! Create a subarray type for the local data in the global data

  call MPI_Type_create_subarray(2,global_size,local_size,local_offset_in_global,&
       &MPI_ORDER_FORTRAN,MPI_DOUBLE_PRECISION,subarray,ierror)
  call MPI_Type_commit(subarray,ierror)
  
  ! Create a subarray type for the local non-halo data in the local full data

  sizes=(/ local_size(1)+2, local_size(2)+2 /)
  subsizes=(/ local_size(1), local_size(2) /)
  begins=(/ 1, 1 /)

  call MPI_Type_create_subarray(2,sizes,subsizes,begins,MPI_ORDER_FORTRAN,&
       &MPI_DOUBLE_PRECISION,nonhaloarray,ierror)
  call MPI_Type_commit(nonhaloarray,ierror)
  
  call MPI_File_set_view(filehandle,offset,MPI_DOUBLE_PRECISION,&
       &subarray,'native',MPI_INFO_NULL,ierror)

  Call MPI_File_write_at_all(filehandle,offset,localTData,1,nonhaloarray,&
       &status,ierror)

  Call MPI_File_close(filehandle,ierror)

  Call MPI_Type_free(nonhaloarray,ierror)
  Call MPI_Type_free(subarray,ierror)

  Call MPI_Finalize(ierror)

End Program MPI_IO_Test
